/**
* @version   $Id: stylepanel.js 26121 2015-01-27 15:22:36Z james $
* @author		RocketTheme http://www.rockettheme.com
* @copyright	Copyright (C) 2007 - 2012 RocketTheme, LLC
* @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 *
 * StylePanel includes also the fabolous ScrollSpy plugin from David Walsh
 * ScrollSpy - David Walsh (http://davidwalsh.name)
 */


((function(){

	this.StylePanel = new Class({

		options:{
			status: 'hide',
			maxWidth: 1288
		},

		initialize: function(element, toggle, options){
			this.setOptions(options);

			this.element = document.getElement(element) || $(element) || null;
			this.toggler = document.getElement(toggle) || $(toggle) || null;
			if (!element) throw new Error('Style Panel "' + element + '" not found in the DOM.');
			if (!toggle) throw new Error('Toggle "' + toggle + '" for Style Panel not fond in the DOM.');

			this.bounds = {
				resize: this.resizeCheck.bind(this)
			};

			this.applyElement = $$(element + ' a.readon')[0];
			this.size = this.element.getSize().size.x;

			if (window.getSize().size.x <= this.options.maxWidth) this.options.status = 'hide';

			this.fx = new Fx.Style(this.element, 'left', {
				duration: 350,
				transition: Fx.Transitions.Quad.easeInOut,
				onComplete: this.onComplete.bind(this, this.element)
			});

			this[this.options.status](null, 'silent');
			this.onComplete(this.element);

			this.attach();
			this.attachBoxes();
		},

		attach: function(){
			this.toggler.addEvent('click', this.toggle.bind(this));
			$$('.preset-item, .pattern-item').each(function(item){
				var click = item['stylepanelitemclick'] || function(event){
					this.click.call(this, event, item);
				}.bind(this);

				item.addEvent('click', click);
			}, this);

			var applyEvent = this.applyElement['stylepanelapplyclick'] || function(event){
				if (event) event.preventDefault();
				this.applyStyles.call(this, this.applyElement);
			}.bind(this);

			this.applyElement.addEvent('click', applyEvent);
			window.addEvent('resize', this.bounds.resize);
		},

		detach: function(){
			this.toggler.removeEvents();
			$$('.preset-item, .pattern-item').each(function(item){
				var click = item['stylepanelitemclick'];
				item.removeEvent('click', click);
			}, this);

			var applyClick = this.applyElement['stylepanelitemclick'];
			this.applyElement.removeEvent('click', applyClick);
			window.removeEvent('resize', this.bounds.resize);
		},

		show: function(event, silent){
			this.fx[silent ? 'set' : 'start'](0);
		},

		hide: function(event, silent){
			for(var rainbow in window['moorainbow']){
				if (window['moorainbow'][rainbow].visible){
					window['moorainbow'][rainbow].hide();
				}
			}
			this.fx[silent ? 'set' : 'start'](-this.size);
		},

		toggle: function(event, silent){
			var current = this.element.getStyle('left').toInt();
			this[!current ? 'hide' : 'show'](event, silent);
		},

		click: function(event, element){
			if (element.className.contains(' style')) return true;

			if (event) event.preventDefault();
			var parent = element.getParent('.presets-block');
			$$('.preset-item, .pattern-item', parent).removeClass('active');
			(function(){ element.addClass('active'); }).delay(10);
		},

		onComplete: function(element){
			var status = element.getStyle('left').toInt();
			this.toggler.removeClass('pig').removeClass('turkey');
			this.toggler.addClass(!status ? 'pig' : 'turkey');
		},

		resizeCheck: function(){
			if (window.getSize().size.x <= this.options.maxWidth && this.element.getStyle('left').toInt() === 0){
				this.hide();
			}

			if (window.getSize().size.x > this.options.maxWidth && this.element.getStyle('left').toInt() < 0){
				this.show();
			}
		},

		attachBoxes: function(){
			GantryColorChooser.add('style-panel-bgcolor', false);
			GantryColorChooser.add('style-panel-accentcolor', false);
		},

		applyStyles: function(element){
			var query = "", pattern = this.element.getElement('.active'),
				bgColor = this.element.getElement('#style-panel-bgcolor'),
				accentColor = this.element.getElement('#style-panel-accentcolor'),
				href = element.getProperty('href');

			query += 'bg-background=' + encodeURIComponent(bgColor.getValue());
			query += '&accent-color=' + encodeURIComponent(accentColor.getValue());
			query +=  (pattern) ? '&bg-pattern=' + pattern.getProperty('rel') : '';

			if (href.contains('&')) href += '&' + query;
			else href += '?' + query;

			window.location = href;
		}

	});

	this.StylePanel.implement(new Options(), new Events());

	var ScrollSpy = new Class({

		/* options */
		options: {
			container: window,
			max: 0,
			min: 0,
			mode: 'vertical'/*,
			onEnter: $empty,
			onLeave: $empty,
			onScroll: $empty,
			onTick: $empty
			*/
		},

		/* initialization */
		initialize: function(options) {
			/* set options */
			this.setOptions(options);
			this.container = $(this.options.container);
			this.enters = this.leaves = 0;
			this.inside = false;

			/* listener */
			var self = this;
			this.listener = function(e) {
				/* if it has reached the level */
				var position = self.container.getSize().scroll,
					xy = position[self.options.mode == 'vertical' ? 'y' : 'x'];
				/* if we reach the minimum and are still below the max... */
				if(xy >= self.options.min && (self.options.max === 0 || xy <= self.options.max)) {
						/* trigger enter event if necessary */
						if(!self.inside) {
							/* record as inside */
							self.inside = true;
							self.enters++;
							/* fire enter event */
							self.fireEvent('onEnter',[position,self.enters,e]);
						}
						/* trigger the "tick", always */
						self.fireEvent('onTick',[position,self.inside,self.enters,self.leaves,e]);
				}
				/* trigger leave */
				else if(self.inside){
					self.inside = false;
					self.leaves++;
					self.fireEvent('onLeave',[position,self.leaves,e]);
				}
				/* fire scroll event */
				self.fireEvent('onScroll',[position,self.inside,self.enters,self.leaves,e]);
			};

			/* make it happen */
			this.addListener();
		},

		/* starts the listener */
		start: function() {
			this.container.addEvent('scroll',this.listener);
		},

		/* stops the listener */
		stop: function() {
			this.container.removeEvent('scroll',this.listener);
		},

		/* legacy */
		addListener: function() {
			this.start();
		}
	});

	ScrollSpy.implement(new Options(), new Events());

	window.addEvent('domready', function(){
		var panel = document.getElement('.style-panel-container'),
			sp = new StylePanel('.style-panel-container', '.style-panel-toggle', {status: 'show'});
			ss = new ScrollSpy({
				min: 150,
				onEnter: function(){ panel.setStyle('position', 'fixed'); },
				onLeave: function(){ panel.setStyle('position', 'absolute'); },
				onTick: function(){
					for(var rainbow in window['moorainbow']){
						if (window['moorainbow'][rainbow].visible){
							window['moorainbow'][rainbow].rePosition();
						}
					}
				}
			});

		window.fireEvent('scroll');
	});

})());
